"""Tests for SecretsAnalyzer."""

from llm_authz_audit.analyzers.secrets import SecretsAnalyzer


class TestSecretsAnalyzer:
    def setup_method(self):
        self.analyzer = SecretsAnalyzer()

    def test_detects_openai_key(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": 'OPENAI_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"'
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) >= 1
        assert findings[0].rule_id == "SEC001"
        assert findings[0].severity.value == "critical"

    def test_suppresses_environ(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": 'OPENAI_KEY = os.environ["OPENAI_API_KEY"]'
        })
        findings = self.analyzer.analyze(ctx)
        sec001 = [f for f in findings if f.rule_id == "SEC001"]
        assert len(sec001) == 0

    def test_suppresses_getenv(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": 'key = os.getenv("OPENAI_API_KEY", "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890")'
        })
        findings = self.analyzer.analyze(ctx)
        sec001 = [f for f in findings if f.rule_id == "SEC001"]
        assert len(sec001) == 0

    def test_detects_anthropic_key(self, make_scan_context):
        ctx = make_scan_context({
            "config.py": 'API_KEY = "sk-ant-abcdefghijklmnopqrstuvwxyz12"'
        })
        findings = self.analyzer.analyze(ctx)
        assert any(f.rule_id == "SEC002" for f in findings)

    def test_detects_huggingface_token(self, make_scan_context):
        ctx = make_scan_context({
            "config.yaml": 'token: hf_abcdefghijklmnopqrstuvwxyz12'
        })
        findings = self.analyzer.analyze(ctx)
        assert any(f.rule_id == "SEC003" for f in findings)

    def test_detects_aws_key(self, make_scan_context):
        ctx = make_scan_context({
            "config.py": 'AWS_KEY = "AKIAIOSFODNN7EXAMPLE"'
        })
        findings = self.analyzer.analyze(ctx)
        assert any(f.rule_id == "SEC004" for f in findings)

    def test_detects_generic_api_key(self, make_scan_context):
        ctx = make_scan_context({
            "settings.py": 'api_key = "abcdefghijklmnop1234567890"'
        })
        findings = self.analyzer.analyze(ctx)
        assert any(f.rule_id == "SEC005" for f in findings)

    def test_clean_file_no_findings(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": '''import os\napi_key = os.environ["API_KEY"]\n'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) == 0

    def test_should_run_with_files(self, make_scan_context):
        ctx = make_scan_context({"app.py": "x = 1"})
        assert self.analyzer.should_run(ctx)

    def test_should_run_no_files(self, make_scan_context):
        ctx = make_scan_context({})
        assert not self.analyzer.should_run(ctx)

    def test_nosec_suppression(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": 'key = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"  # nosec'
        })
        findings = self.analyzer.analyze(ctx)
        sec001 = [f for f in findings if f.rule_id == "SEC001"]
        assert len(sec001) == 0



"""ASCII art intro banner for console output."""

from __future__ import annotations

from typing import TYPE_CHECKING

from llm_authz_audit import __version__

if TYPE_CHECKING:
    from pathlib import Path

    from rich.console import Console


_ART = """\
  [bold #10b981]╦   ╦   ╔╦╗[/bold #10b981]
  [bold #10b981]║   ║   ║║║[/bold #10b981]
  [bold #10b981]╩═╝ ╩═╝ ╩ ╩[/bold #10b981]   [bold #e2e8f0]authz[/bold #e2e8f0][#64748b]-[/#64748b][bold #10b981]audit[/bold #10b981]"""

_TAGLINE = "  Static Security Analyzer for LLM Applications"
_URL = "  https://github.com/aiauthz/llm-authz-audit"


def print_banner(
    console: Console,
    *,
    target: Path,
    analyzers_loaded: int,
    rules_loaded: int,
    fail_on: str = "high",
    exclude_patterns: list[str] | None = None,
    config_file: Path | None = None,
    suppress_file: Path | None = None,
    min_confidence: str | None = None,
) -> None:
    """Render the intro banner to *console*."""
    console.print(_ART, highlight=False)
    console.print(f"[dim]{_TAGLINE}  v{__version__}[/dim]", highlight=False)
    console.print(f"[dim underline]{_URL}[/dim underline]", highlight=False)
    console.print()
    console.print(f"  [dim]Target:[/dim]    [dim bold]{target}[/dim bold]", highlight=False)
    console.print(
        f"  [dim]Analyzers:[/dim] [dim bold]{analyzers_loaded} loaded[/dim bold]"
        f" [dim]|[/dim] [dim]Rules:[/dim] [dim bold]{rules_loaded} loaded[/dim bold]",
        highlight=False,
    )
    console.print(f"  [dim]Fail on:[/dim]  [dim bold]{fail_on}[/dim bold]", highlight=False)
    if config_file:
        console.print(f"  [dim]Config:[/dim]   [dim bold]{config_file}[/dim bold]", highlight=False)
    if suppress_file:
        console.print(f"  [dim]Suppress:[/dim] [dim bold]{suppress_file}[/dim bold]", highlight=False)
    if min_confidence:
        console.print(f"  [dim]Min conf:[/dim] [dim bold]{min_confidence}[/dim bold]", highlight=False)
    if exclude_patterns:
        console.print(
            f"  [dim]Exclude:[/dim]  [dim bold]{', '.join(exclude_patterns)}[/dim bold]",
            highlight=False,
        )
    console.print()



"""Tests for Finding, Severity, Confidence."""

from llm_authz_audit.core.finding import Confidence, Finding, Severity


class TestSeverity:
    def test_ordering(self):
        assert Severity.LOW < Severity.MEDIUM < Severity.HIGH < Severity.CRITICAL

    def test_from_string(self):
        assert Severity("critical") == Severity.CRITICAL
        assert Severity("low") == Severity.LOW

    def test_comparisons(self):
        assert Severity.HIGH >= Severity.MEDIUM
        assert Severity.LOW <= Severity.HIGH
        assert not Severity.LOW > Severity.HIGH


class TestFinding:
    def _make_finding(self, **overrides) -> Finding:
        defaults = dict(
            rule_id="SEC001",
            title="Test finding",
            severity=Severity.HIGH,
            confidence=Confidence.HIGH,
            file_path="app.py",
            line_number=10,
            snippet="sk-proj-abc123",
            description="Hardcoded API key",
            remediation="Use env vars",
            analyzer="SecretsAnalyzer",
        )
        defaults.update(overrides)
        return Finding(**defaults)

    def test_unique_key(self):
        f = self._make_finding()
        assert f.unique_key == ("SEC001", "app.py", 10)

    def test_unique_key_dedup(self):
        f1 = self._make_finding()
        f2 = self._make_finding()
        assert f1.unique_key == f2.unique_key

    def test_to_dict(self):
        f = self._make_finding(owasp_llm="LLM06")
        d = f.to_dict()
        assert d["rule_id"] == "SEC001"
        assert d["severity"] == "high"
        assert d["confidence"] == "high"
        assert d["owasp_llm"] == "LLM06"
        assert isinstance(d["metadata"], dict)

    def test_owasp_llm_default_none(self):
        f = self._make_finding()
        assert f.owasp_llm is None


class TestConfidence:
    def test_ai_verified(self):
        assert Confidence.AI_VERIFIED.value == "ai_verified"

    def test_ordering(self):
        assert Confidence.LOW < Confidence.MEDIUM < Confidence.HIGH < Confidence.AI_VERIFIED

    def test_ge(self):
        assert Confidence.HIGH >= Confidence.MEDIUM
        assert Confidence.HIGH >= Confidence.HIGH

    def test_le(self):
        assert Confidence.LOW <= Confidence.HIGH
        assert Confidence.MEDIUM <= Confidence.MEDIUM

    def test_gt_false(self):
        assert not Confidence.LOW > Confidence.HIGH

    def test_not_comparable_to_other_types(self):
        assert Confidence.HIGH.__lt__("high") is NotImplemented



"""Tests for OutputFilteringAnalyzer."""

from llm_authz_audit.analyzers.output_filtering import OutputFilteringAnalyzer


class TestOutputFilteringAnalyzer:
    def setup_method(self):
        self.analyzer = OutputFilteringAnalyzer()

    def test_detects_unfiltered_output(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": '''
from openai import OpenAI

client = OpenAI()
response = client.chat.completions.create(model="gpt-4", messages=[])
answer = response.content
return answer
'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) >= 1
        assert findings[0].rule_id == "OF001"

    def test_suppresses_with_filtering(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": '''
from openai import OpenAI

client = OpenAI()
response = client.chat.completions.create(model="gpt-4", messages=[])
answer = response.content
filtered = sanitize(answer)
return filtered
'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) == 0

    def test_should_run_openai(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": 'from openai import OpenAI\n'
        })
        assert self.analyzer.should_run(ctx)

    def test_should_not_run_no_llm_lib(self, make_scan_context):
        ctx = make_scan_context({"app.py": 'print("hello")'})
        assert not self.analyzer.should_run(ctx)



"""Integration tests for the full scan pipeline."""


from llm_authz_audit.core.config import ToolConfig
from llm_authz_audit.core.engine import ScanEngine
from llm_authz_audit.core.finding import Severity
from llm_authz_audit.output.formatter import FormatterFactory


class TestEnginePipeline:
    def test_full_pipeline_vulnerable_project(self, tmp_path):
        # Create a mini vulnerable project
        (tmp_path / "app.py").write_text('''
from fastapi import FastAPI
app = FastAPI()

API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"

@app.post("/chat")
async def chat(message: str):
    return {"response": "hello"}
''')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()

        assert result.files_scanned >= 1
        assert len(result.findings) >= 2  # At least secrets + endpoint
        assert result.exit_code == 1

        # Verify findings are sorted by severity
        for i in range(len(result.findings) - 1):
            assert result.findings[i].severity >= result.findings[i + 1].severity

    def test_full_pipeline_clean_project(self, tmp_path):
        (tmp_path / "app.py").write_text('''
import os
api_key = os.environ["API_KEY"]
print("clean project")
''')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()

        assert result.exit_code == 0
        assert len(result.findings) == 0

    def test_json_output(self, tmp_path):
        import json

        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()

        formatter = FormatterFactory.get("json")
        output = formatter.format(result)

        # Should be valid JSON
        data = json.loads(output)
        assert "findings" in data
        assert "summary" in data
        assert data["summary"]["total_findings"] > 0

    def test_console_output(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()

        formatter = FormatterFactory.get("console")
        output = formatter.format(result)
        assert "SEC001" in output
        assert "finding" in output.lower()

    def test_fail_on_critical_only(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path, fail_on=Severity.CRITICAL)
        engine = ScanEngine(config)
        result = engine.scan()
        # SEC001 is critical
        assert result.exit_code == 1

    def test_fail_on_low_catches_everything(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path, fail_on=Severity.LOW)
        engine = ScanEngine(config)
        result = engine.scan()
        assert result.exit_code == 1



"""Tests for ScanEngine."""


from llm_authz_audit.core.config import ToolConfig
from llm_authz_audit.core.engine import ScanEngine
from llm_authz_audit.core.finding import Severity


class TestScanEngine:
    def test_scan_empty_dir(self, tmp_path):
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()
        assert result.files_scanned == 0
        assert result.exit_code == 0
        assert result.findings == []

    def test_scan_detects_hardcoded_key(self, tmp_path):
        app_py = tmp_path / "app.py"
        app_py.write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()
        assert len(result.findings) > 0
        assert any(f.rule_id == "SEC001" for f in result.findings)

    def test_scan_exit_code_on_high(self, tmp_path):
        app_py = tmp_path / "app.py"
        app_py.write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path, fail_on=Severity.HIGH)
        engine = ScanEngine(config)
        result = engine.scan()
        # SEC001 is critical, which is >= high
        assert result.exit_code == 1

    def test_scan_exit_code_zero_below_threshold(self, tmp_path):
        app_py = tmp_path / "app.py"
        app_py.write_text('x = 1\n')
        config = ToolConfig(target_path=tmp_path, fail_on=Severity.HIGH)
        engine = ScanEngine(config)
        result = engine.scan()
        assert result.exit_code == 0

    def test_scan_with_exclude(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        (tmp_path / "safe.py").write_text('x = 1\n')
        config = ToolConfig(target_path=tmp_path, exclude_patterns=["app.py"])
        engine = ScanEngine(config)
        result = engine.scan()
        assert not any(f.file_path == "app.py" for f in result.findings)

    def test_scan_deduplicates(self, tmp_path):
        app_py = tmp_path / "app.py"
        app_py.write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()
        keys = [f.unique_key for f in result.findings]
        assert len(keys) == len(set(keys))

    def test_scan_with_enabled_analyzers(self, tmp_path):
        (tmp_path / "app.py").write_text('x = 1\n')
        config = ToolConfig(
            target_path=tmp_path,
            enabled_analyzers=["SecretsAnalyzer"],
        )
        engine = ScanEngine(config)
        result = engine.scan()
        assert "SecretsAnalyzer" in result.analyzers_run
        assert all(a not in result.analyzers_run for a in ["EndpointAnalyzer", "ToolRBACAnalyzer"])

    def test_analyzers_property(self, tmp_path):
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        assert len(engine.analyzers) > 0

    def test_summary(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        config = ToolConfig(target_path=tmp_path)
        engine = ScanEngine(config)
        result = engine.scan()
        summary = result.summary
        assert isinstance(summary, dict)
        assert "critical" in summary



"""Tests for ScanContext and FileEntry."""


from llm_authz_audit.core.context import FileEntry


class TestFileEntry:
    def test_lazy_content_loading(self, tmp_path):
        f = tmp_path / "test.py"
        f.write_text("print('hello')")
        entry = FileEntry(path=f, relative_path="test.py")
        assert entry._content is None
        assert entry.content == "print('hello')"
        assert entry._content is not None

    def test_ast_tree_for_python(self, tmp_path):
        f = tmp_path / "test.py"
        f.write_text("x = 1")
        entry = FileEntry(path=f, relative_path="test.py")
        tree = entry.ast_tree
        assert tree is not None

    def test_ast_tree_none_for_non_python(self, tmp_path):
        f = tmp_path / "test.yaml"
        f.write_text("key: value")
        entry = FileEntry(path=f, relative_path="test.yaml")
        assert entry.ast_tree is None

    def test_ast_tree_none_for_syntax_error(self, tmp_path):
        f = tmp_path / "bad.py"
        f.write_text("def (broken:")
        entry = FileEntry(path=f, relative_path="bad.py")
        assert entry.ast_tree is None

    def test_suffix(self, tmp_path):
        f = tmp_path / "test.py"
        f.write_text("")
        entry = FileEntry(path=f, relative_path="test.py")
        assert entry.suffix == ".py"


class TestScanContext:
    def test_python_files(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": "x = 1",
            "config.yaml": "key: val",
            "utils.py": "y = 2",
        })
        assert len(ctx.python_files()) == 2

    def test_config_files(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": "x = 1",
            "config.yaml": "key: val",
            "settings.json": "{}",
            "setup.toml": "[tool]",
        })
        assert len(ctx.config_files()) == 3

    def test_files_matching(self, make_scan_context):
        ctx = make_scan_context({
            "src/app.py": "x = 1",
            "src/utils.py": "y = 2",
            "tests/test_app.py": "z = 3",
        })
        assert len(ctx.files_matching("src/*.py")) == 2

    def test_files_matching_any(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": "",
            "config.yaml": "",
            "data.json": "",
        })
        results = ctx.files_matching_any(["*.py", "*.yaml"])
        assert len(results) == 2



"""CF*: Secrets in prompt templates/chains."""

from __future__ import annotations

import re

from llm_authz_audit.analyzers import register_analyzer
from llm_authz_audit.analyzers.base import BaseAnalyzer
from llm_authz_audit.core.context import ScanContext
from llm_authz_audit.core.finding import Confidence, Finding, Severity
from llm_authz_audit.parsers.python_ast import find_fstring_variables

_SENSITIVE_NAMES = {
    "password", "passwd", "pwd", "secret", "token",
    "api_key", "apikey", "credential", "private_key",
    "access_key", "secret_key", "auth_token",
}


@register_analyzer
class CredentialForwardingAnalyzer(BaseAnalyzer):
    name = "CredentialForwardingAnalyzer"
    description = "Detects credentials being forwarded to LLM via prompt templates."

    def should_run(self, context: ScanContext) -> bool:
        return len(context.python_files()) > 0

    def analyze(self, context: ScanContext) -> list[Finding]:
        findings: list[Finding] = []
        for file_entry in context.python_files():
            tree = file_entry.ast_tree
            if tree is None:
                continue
            content_lines = file_entry.content.splitlines()

            # Check f-strings for sensitive variables
            for var_name, lineno in find_fstring_variables(tree):
                if self._is_sensitive(var_name):
                    line = content_lines[lineno - 1] if lineno <= len(content_lines) else ""
                    # Check if this is in a prompt-like context
                    if self._is_prompt_context(content_lines, lineno):
                        if any(s in line for s in ("mask", "redact", "# nosec")):
                            continue
                        findings.append(Finding(
                            rule_id="CF001",
                            title="Credential in prompt template",
                            severity=Severity.CRITICAL,
                            confidence=Confidence.MEDIUM,
                            file_path=file_entry.relative_path,
                            line_number=lineno,
                            snippet=line.strip(),
                            description=f"Sensitive variable '{var_name}' used in what appears to be a prompt template.",
                            remediation="Never include credentials in prompt templates. Use sanitized references.",
                            analyzer=self.name,
                            owasp_llm="LLM06",
                        ))

            # Check .format() calls with sensitive vars
            for lineno, line in enumerate(content_lines, start=1):
                if ".format(" in line:
                    for name in _SENSITIVE_NAMES:
                        if re.search(rf'\b{name}\b', line, re.IGNORECASE):
                            if self._is_prompt_context(content_lines, lineno):
                                if any(s in line for s in ("mask", "redact", "# nosec")):
                                    continue
                                findings.append(Finding(
                                    rule_id="CF001",
                                    title="Credential in prompt template",
                                    severity=Severity.CRITICAL,
                                    confidence=Confidence.MEDIUM,
                                    file_path=file_entry.relative_path,
                                    line_number=lineno,
                                    snippet=line.strip(),
                                    description=f"Sensitive value '{name}' in .format() call in prompt context.",
                                    remediation="Never include credentials in prompt templates.",
                                    analyzer=self.name,
                                    owasp_llm="LLM06",
                                ))
                                break
        return findings

    def _is_sensitive(self, name: str) -> bool:
        name_lower = name.lower()
        return any(s in name_lower for s in _SENSITIVE_NAMES)

    def _is_prompt_context(self, lines: list[str], lineno: int) -> bool:
        start = max(0, lineno - 5)
        end = min(len(lines), lineno + 5)
        context = "\n".join(lines[start:end]).lower()
        prompt_indicators = (
            "prompt", "template", "system_message", "human_message",
            "user_message", "llm", "chat", "completion", "instruction",
        )
        return any(pi in context for pi in prompt_indicators)



"""OF*: Missing PII/output filtering."""

from __future__ import annotations

import re

from llm_authz_audit.analyzers import register_analyzer
from llm_authz_audit.analyzers.base import BaseAnalyzer
from llm_authz_audit.core.context import ScanContext
from llm_authz_audit.core.finding import Confidence, Finding, Severity
from llm_authz_audit.parsers.python_ast import has_import

_OUTPUT_PATTERNS = [
    r"\.content\b",
    r"\.text\b",
    r"response\[.message.\]",
    r"completion\.choices",
    r"\.generations\b",
]

_FILTER_INDICATORS = {
    "filter", "sanitize", "redact", "pii", "mask",
    "guard", "moderate", "content_filter", "guardrail",
}


@register_analyzer
class OutputFilteringAnalyzer(BaseAnalyzer):
    name = "OutputFilteringAnalyzer"
    description = "Detects LLM output used without content filtering or PII redaction."

    def should_run(self, context: ScanContext) -> bool:
        for f in context.python_files():
            tree = f.ast_tree
            if tree and (
                has_import(tree, "openai")
                or has_import(tree, "anthropic")
                or has_import(tree, "langchain")
                or has_import(tree, "llama_index")
            ):
                return True
        return False

    def analyze(self, context: ScanContext) -> list[Finding]:
        findings: list[Finding] = []
        for file_entry in context.python_files():
            tree = file_entry.ast_tree
            if tree is None:
                continue
            if not (
                has_import(tree, "openai")
                or has_import(tree, "anthropic")
                or has_import(tree, "langchain")
                or has_import(tree, "llama_index")
            ):
                continue

            content = file_entry.content
            # Check if file has any filtering
            if any(fi in content for fi in _FILTER_INDICATORS):
                continue

            content_lines = content.splitlines()
            for lineno, line in enumerate(content_lines, start=1):
                for pattern in _OUTPUT_PATTERNS:
                    if re.search(pattern, line):
                        # Only flag if it looks like LLM output access
                        if any(kw in line for kw in ("response", "completion", "result", "output", "answer", "reply")):
                            findings.append(Finding(
                                rule_id="OF001",
                                title="LLM output without filtering",
                                severity=Severity.MEDIUM,
                                confidence=Confidence.LOW,
                                file_path=file_entry.relative_path,
                                line_number=lineno,
                                snippet=line.strip(),
                                description="LLM output used without content filtering or PII redaction.",
                                remediation="Add output filtering for PII, sensitive data, and harmful content.",
                                analyzer=self.name,
                                owasp_llm="LLM02",
                            ))
                            break  # one finding per line
        return findings



"""Tests for MCPPermissionAnalyzer."""

from llm_authz_audit.analyzers.mcp_permissions import MCPPermissionAnalyzer


class TestMCPPermissionAnalyzer:
    def setup_method(self):
        self.analyzer = MCPPermissionAnalyzer()

    def test_detects_unauthenticated_server(self, make_scan_context):
        ctx = make_scan_context({
            "mcp.json": '''{
  "mcpServers": {
    "myserver": {
      "url": "http://localhost:8080",
      "env": {}
    }
  }
}'''
        })
        findings = self.analyzer.analyze(ctx)
        mcp002 = [f for f in findings if f.rule_id == "MCP002"]
        assert len(mcp002) == 1

    def test_suppresses_authenticated_server(self, make_scan_context):
        ctx = make_scan_context({
            "mcp.json": '''{
  "mcpServers": {
    "myserver": {
      "url": "http://localhost:8080",
      "headers": {"Authorization": "Bearer token"}
    }
  }
}'''
        })
        findings = self.analyzer.analyze(ctx)
        mcp002 = [f for f in findings if f.rule_id == "MCP002"]
        assert len(mcp002) == 0

    def test_detects_wildcard_tools(self, make_scan_context):
        ctx = make_scan_context({
            "mcp.json": '''{
  "mcpServers": {
    "myserver": {
      "tools": ["*"]
    }
  }
}'''
        })
        findings = self.analyzer.analyze(ctx)
        mcp003 = [f for f in findings if f.rule_id == "MCP003"]
        assert len(mcp003) >= 1

    def test_should_run_mcp_json(self, make_scan_context):
        ctx = make_scan_context({"mcp.json": "{}"})
        assert self.analyzer.should_run(ctx)

    def test_should_run_claude_desktop_config(self, make_scan_context):
        ctx = make_scan_context({"claude_desktop_config.json": "{}"})
        assert self.analyzer.should_run(ctx)

    def test_should_not_run_no_mcp(self, make_scan_context):
        ctx = make_scan_context({"app.py": 'print("hello")'})
        assert not self.analyzer.should_run(ctx)



"""Tests for CredentialForwardingAnalyzer."""

from llm_authz_audit.analyzers.credential_forwarding import CredentialForwardingAnalyzer


class TestCredentialForwardingAnalyzer:
    def setup_method(self):
        self.analyzer = CredentialForwardingAnalyzer()

    def test_detects_password_in_fstring_prompt(self, make_scan_context):
        ctx = make_scan_context({
            "chain.py": '''
prompt_template = f"User password is {password}, please verify"
llm_response = chat(prompt_template)
'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) >= 1
        assert findings[0].rule_id == "CF001"

    def test_detects_secret_in_format(self, make_scan_context):
        ctx = make_scan_context({
            "chain.py": '''
prompt = "The api_key is {secret}".format(secret=api_secret)
completion = llm.chat(prompt)
'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) >= 1
        assert findings[0].rule_id == "CF001"

    def test_no_false_positive_safe_code(self, make_scan_context):
        ctx = make_scan_context({
            "app.py": '''
name = "Alice"
greeting = f"Hello, {name}!"
print(greeting)
'''
        })
        findings = self.analyzer.analyze(ctx)
        assert len(findings) == 0

    def test_should_run_with_python_files(self, make_scan_context):
        ctx = make_scan_context({"app.py": 'x = 1'})
        assert self.analyzer.should_run(ctx)

    def test_should_not_run_no_python_files(self, make_scan_context):
        ctx = make_scan_context({"config.yaml": "key: val"})
        assert not self.analyzer.should_run(ctx)



"""Tests for PromptInjectionAnalyzer."""

import pytest

from llm_authz_audit.analyzers.prompt_injection import PromptInjectionAnalyzer


@pytest.fixture
def analyzer():
    return PromptInjectionAnalyzer()


class TestShouldRun:
    def test_true_for_openai(self, analyzer, make_scan_context):
        ctx = make_scan_context({"app.py": "import openai\n"})
        assert analyzer.should_run(ctx)

    def test_true_for_anthropic(self, analyzer, make_scan_context):
        ctx = make_scan_context({"app.py": "import anthropic\n"})
        assert analyzer.should_run(ctx)

    def test_true_for_langchain(self, analyzer, make_scan_context):
        ctx = make_scan_context({"app.py": "from langchain import LLMChain\n"})
        assert analyzer.should_run(ctx)

    def test_false_for_no_llm(self, analyzer, make_scan_context):
        ctx = make_scan_context({"app.py": "import os\n"})
        assert not analyzer.should_run(ctx)


class TestPI001:
    def test_fstring_user_input(self, analyzer, make_scan_context):
        code = '''\
import openai
def chat(user_input):
    prompt = f"Answer: {user_input}"
    openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi001 = [f for f in findings if f.rule_id == "PI001"]
        assert len(pi001) >= 1
        assert pi001[0].severity.value == "critical"
        assert pi001[0].owasp_llm == "LLM01"

    def test_format_with_query(self, analyzer, make_scan_context):
        code = '''\
import openai
def search(query):
    template = "Search for: {}"
    prompt = template.format(query)
    openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi001 = [f for f in findings if f.rule_id == "PI001"]
        assert len(pi001) >= 1

    def test_suppressed_with_sanitize(self, analyzer, make_scan_context):
        code = '''\
import openai
def chat(user_input):
    clean = sanitize(user_input)
    prompt = f"Answer: {clean}"
    openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi001 = [f for f in findings if f.rule_id == "PI001"]
        assert len(pi001) == 0


class TestPI002:
    def test_string_concat_prompt(self, analyzer, make_scan_context):
        code = '''\
import openai
def build():
    prompt = "System: " + user_data
    openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi002 = [f for f in findings if f.rule_id == "PI002"]
        assert len(pi002) >= 1
        assert pi002[0].severity.value == "high"

    def test_augmented_assign(self, analyzer, make_scan_context):
        code = '''\
import openai
def build():
    prompt = "System instruction"
    prompt += extra_data
    openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi002 = [f for f in findings if f.rule_id == "PI002"]
        assert len(pi002) >= 1


class TestPI003:
    def test_missing_delimiter(self, analyzer, make_scan_context):
        code = '''\
import openai
def chat(user_input):
    result = openai.ChatCompletion.create(prompt=user_input)
    return result
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi003 = [f for f in findings if f.rule_id == "PI003"]
        assert len(pi003) >= 1
        assert pi003[0].confidence.value == "low"

    def test_suppressed_with_backtick_delimiter(self, analyzer, make_scan_context):
        code = '''\
import openai
def chat(user_input):
    prompt = f"Analyze:\\n```\\n{user_input}\\n```"
    result = openai.ChatCompletion.create(prompt=prompt)
    return result
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        pi003 = [f for f in findings if f.rule_id == "PI003"]
        assert len(pi003) == 0


class TestCleanCode:
    def test_no_false_positives(self, analyzer, make_scan_context):
        code = '''\
import openai
def safe_call():
    prompt = "Static instruction with no user input"
    return openai.ChatCompletion.create(prompt=prompt)
'''
        ctx = make_scan_context({"app.py": code})
        findings = analyzer.analyze(ctx)
        assert len(findings) == 0

    def test_non_python_ignored(self, analyzer, make_scan_context):
        ctx = make_scan_context({"readme.md": "# Prompt injection user_input openai"})
        findings = analyzer.analyze(ctx)
        assert len(findings) == 0



"""Suppression loading and matching."""

from __future__ import annotations

from dataclasses import dataclass
from fnmatch import fnmatch
from pathlib import Path
from typing import TYPE_CHECKING

import yaml

if TYPE_CHECKING:
    from llm_authz_audit.core.finding import Finding


@dataclass
class Suppression:
    rule_id: str | None
    file_pattern: str | None
    reason: str

    def matches(self, finding: Finding) -> bool:
        if self.rule_id and self.rule_id != finding.rule_id:
            return False
        if self.file_pattern and not fnmatch(finding.file_path, self.file_pattern):
            return False
        return True


class SuppressionLoader:
    @staticmethod
    def load(path: Path) -> list[Suppression]:
        if not path.exists():
            return []
        with open(path, encoding="utf-8") as f:
            data = yaml.safe_load(f)
        if not data or "suppressions" not in data:
            return []
        result: list[Suppression] = []
        for entry in data["suppressions"]:
            rule_id = entry.get("rule_id")
            file_pattern = entry.get("file_pattern")
            reason = entry.get("reason", "")
            if not rule_id and not file_pattern:
                continue
            if not reason:
                continue
            result.append(Suppression(rule_id=rule_id, file_pattern=file_pattern, reason=reason))
        return result


def apply_suppressions(findings: list[Finding], suppressions: list[Suppression]) -> list[Finding]:
    return [f for f in findings if not any(s.matches(f) for s in suppressions)]



"""Tool configuration."""

from __future__ import annotations

from dataclasses import dataclass, field
from pathlib import Path

from llm_authz_audit.core.finding import Confidence, Severity


@dataclass
class ToolConfig:
    target_path: Path = field(default_factory=lambda: Path("."))
    output_format: str = "console"
    fail_on: Severity = Severity.HIGH
    enabled_analyzers: list[str] | None = None
    exclude_patterns: list[str] = field(default_factory=list)
    suppress_file: Path | None = None
    verbose: bool = False
    ai_enabled: bool = False
    ai_provider: str = "anthropic"
    ai_model: str = "claude-sonnet-4-5-20250929"
    config_file: Path | None = None
    extra_rule_dirs: list[Path] = field(default_factory=list)
    ai_max_findings: int = 20
    diff_ref: str | None = None
    min_confidence: Confidence | None = None

    @classmethod
    def from_dict(cls, data: dict) -> ToolConfig:
        if "fail_on" in data and isinstance(data["fail_on"], str):
            data["fail_on"] = Severity(data["fail_on"])
        if "target_path" in data and isinstance(data["target_path"], str):
            data["target_path"] = Path(data["target_path"])
        if "suppress_file" in data and isinstance(data["suppress_file"], str):
            data["suppress_file"] = Path(data["suppress_file"])
        if "config_file" in data and isinstance(data["config_file"], str):
            data["config_file"] = Path(data["config_file"])
        if "extra_rule_dirs" in data:
            data["extra_rule_dirs"] = [Path(p) for p in data["extra_rule_dirs"]]
        if "min_confidence" in data and isinstance(data["min_confidence"], str):
            data["min_confidence"] = Confidence(data["min_confidence"])
        return cls(**{k: v for k, v in data.items() if k in cls.__dataclass_fields__})



"""Pydantic validation for YAML rule files."""

from __future__ import annotations

from pydantic import BaseModel, field_validator


class RuleSchema(BaseModel):
    id: str
    title: str
    severity: str
    pattern: str
    file_types: list[str] = ["*.py"]
    suppress_if: list[str] = []
    owasp_llm: str | None = None
    remediation: str = ""
    description: str = ""
    analyzer: str = ""

    @field_validator("severity")
    @classmethod
    def validate_severity(cls, v: str) -> str:
        allowed = {"critical", "high", "medium", "low"}
        if v.lower() not in allowed:
            raise ValueError(f"severity must be one of {allowed}, got '{v}'")
        return v.lower()

    @field_validator("id")
    @classmethod
    def validate_id(cls, v: str) -> str:
        if not v or len(v) < 3:
            raise ValueError(f"Rule id must be at least 3 characters, got '{v}'")
        return v


class RuleFileSchema(BaseModel):
    rules: list[RuleSchema]


def validate_rule_file(data: dict) -> RuleFileSchema:
    return RuleFileSchema(**data)



"""Base formatter and factory."""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from llm_authz_audit.core.engine import ScanResult


class BaseFormatter(ABC):
    @abstractmethod
    def format(self, result: ScanResult) -> str:
        """Format scan results into a string."""


class FormatterFactory:
    _formatters: dict[str, type[BaseFormatter]] = {}

    @classmethod
    def register(cls, name: str, formatter_cls: type[BaseFormatter]) -> None:
        cls._formatters[name] = formatter_cls

    @classmethod
    def get(cls, name: str) -> BaseFormatter:
        if name not in cls._formatters:
            # Trigger imports
            from llm_authz_audit.output import console, json_output, sarif  # noqa: F401
        if name not in cls._formatters:
            raise ValueError(f"Unknown formatter: {name}. Available: {list(cls._formatters.keys())}")
        return cls._formatters[name]()

    @classmethod
    def available(cls) -> list[str]:
        from llm_authz_audit.output import console, json_output, sarif  # noqa: F401
        return list(cls._formatters.keys())



"""Tests for --diff mode: get_diff_files and FileDiscovery filtering."""

from unittest.mock import patch

from llm_authz_audit.core.discovery import FileDiscovery, get_diff_files


class TestGetDiffFiles:
    def test_returns_files_from_git(self, tmp_path):
        with patch("subprocess.run") as mock_run:
            mock_run.return_value.returncode = 0
            mock_run.return_value.stdout = "app.py\nsrc/utils.py\n"
            result = get_diff_files(tmp_path, "HEAD~1")
        assert result == {"app.py", "src/utils.py"}

    def test_empty_on_failure(self, tmp_path):
        with patch("subprocess.run") as mock_run:
            mock_run.return_value.returncode = 1
            mock_run.return_value.stdout = ""
            result = get_diff_files(tmp_path, "HEAD~1")
        assert result == set()

    def test_empty_on_timeout(self, tmp_path):
        import subprocess
        with patch("subprocess.run", side_effect=subprocess.TimeoutExpired("git", 30)):
            result = get_diff_files(tmp_path, "HEAD~1")
        assert result == set()

    def test_empty_on_no_git(self, tmp_path):
        with patch("subprocess.run", side_effect=FileNotFoundError):
            result = get_diff_files(tmp_path, "HEAD~1")
        assert result == set()

    def test_strips_whitespace(self, tmp_path):
        with patch("subprocess.run") as mock_run:
            mock_run.return_value.returncode = 0
            mock_run.return_value.stdout = "  app.py  \n\n  utils.py  \n"
            result = get_diff_files(tmp_path, "main")
        assert result == {"app.py", "utils.py"}


class TestFileDiscoveryDiffFilter:
    def test_filters_to_diff_files(self, tmp_path):
        (tmp_path / "app.py").write_text("x = 1\n")
        (tmp_path / "other.py").write_text("y = 2\n")

        discovery = FileDiscovery(tmp_path, diff_files={"app.py"})
        files = discovery.discover()

        paths = {f.relative_path for f in files}
        assert "app.py" in paths
        assert "other.py" not in paths

    def test_no_filter_when_none(self, tmp_path):
        (tmp_path / "app.py").write_text("x = 1\n")
        (tmp_path / "other.py").write_text("y = 2\n")

        discovery = FileDiscovery(tmp_path, diff_files=None)
        files = discovery.discover()

        paths = {f.relative_path for f in files}
        assert "app.py" in paths
        assert "other.py" in paths

    def test_empty_diff_returns_empty(self, tmp_path):
        (tmp_path / "app.py").write_text("x = 1\n")

        discovery = FileDiscovery(tmp_path, diff_files=set())
        files = discovery.discover()
        assert files == []



"""Scan context and file entry models."""

from __future__ import annotations

import ast
from dataclasses import dataclass, field
from fnmatch import fnmatch
from pathlib import Path

from llm_authz_audit.core.config import ToolConfig


@dataclass
class FileEntry:
    path: Path
    relative_path: str
    _content: str | None = field(default=None, repr=False)
    _ast_tree: ast.Module | None = field(default=None, repr=False)

    @property
    def content(self) -> str:
        if self._content is None:
            try:
                self._content = self.path.read_text(encoding="utf-8")
            except (OSError, UnicodeDecodeError):
                self._content = ""
        return self._content

    @property
    def ast_tree(self) -> ast.Module | None:
        if self._ast_tree is None and self.path.suffix == ".py":
            try:
                self._ast_tree = ast.parse(self.content, filename=str(self.path))
            except SyntaxError:
                return None
        return self._ast_tree

    @property
    def suffix(self) -> str:
        return self.path.suffix


@dataclass
class ScanContext:
    target_path: Path
    files: list[FileEntry]
    config: ToolConfig

    def python_files(self) -> list[FileEntry]:
        return [f for f in self.files if f.suffix == ".py"]

    def config_files(self) -> list[FileEntry]:
        config_suffixes = {".yaml", ".yml", ".json", ".toml", ".env", ".ini", ".cfg"}
        return [f for f in self.files if f.suffix in config_suffixes]

    def files_matching(self, pattern: str) -> list[FileEntry]:
        return [f for f in self.files if fnmatch(f.relative_path, pattern)]

    def files_matching_any(self, patterns: list[str]) -> list[FileEntry]:
        return [
            f for f in self.files
            if any(fnmatch(f.relative_path, p) for p in patterns)
        ]



"""AI-powered deep analysis — runs after static analysis to refine findings."""

from __future__ import annotations

import re
from typing import TYPE_CHECKING

from llm_authz_audit.core.finding import Confidence, Finding
from llm_authz_audit.llm.prompts import SYSTEM_PROMPT, build_finding_review_prompt
from llm_authz_audit.llm.providers import get_provider

if TYPE_CHECKING:
    from llm_authz_audit.core.engine import ScanEngine, ScanResult
    from llm_authz_audit.llm.providers import LLMProvider


class AIAnalyzer:
    """Refines static analysis findings using LLM reasoning."""

    def __init__(
        self,
        provider: str = "anthropic",
        model: str = "claude-sonnet-4-5-20250929",
        llm_client: LLMProvider | None = None,
    ) -> None:
        self._provider = llm_client or get_provider(provider, model)

    def refine(self, result: ScanResult, engine: ScanEngine, max_findings: int = 20) -> ScanResult:
        """Review each finding with the LLM and update confidence.

        Only the top *max_findings* by severity are sent to the LLM.
        Remaining findings are kept as-is.
        """
        # Sort by severity desc so critical/high get reviewed first
        sorted_findings = sorted(result.findings, key=lambda f: f.severity, reverse=True)
        to_review = sorted_findings[:max_findings]
        skipped = sorted_findings[max_findings:]

        refined_findings: list[Finding] = []

        for finding in to_review:
            context = self._get_context(finding, engine)
            prompt = build_finding_review_prompt(
                rule_id=finding.rule_id,
                title=finding.title,
                severity=finding.severity.value,
                file_path=finding.file_path,
                line_number=finding.line_number,
                snippet=finding.snippet,
                description=finding.description,
                context=context,
            )

            try:
                response = self._provider.complete(prompt, system=SYSTEM_PROMPT)
                verdict, confidence, explanation = self._parse_response(response)

                if verdict == "FALSE_POSITIVE":
                    continue  # Drop false positives

                finding.confidence = Confidence.AI_VERIFIED
                finding.metadata["ai_explanation"] = explanation
                finding.metadata["ai_confidence"] = confidence
            except Exception:
                # If AI fails, keep original finding
                pass

            refined_findings.append(finding)

        refined_findings.extend(skipped)
        result.findings = refined_findings
        return result

    def _get_context(self, finding: Finding, engine: ScanEngine) -> str:
        """Get surrounding code context for a finding."""

        file_path = engine.config.target_path / finding.file_path
        if not file_path.exists():
            return finding.snippet

        try:
            lines = file_path.read_text(encoding="utf-8").splitlines()
        except (OSError, UnicodeDecodeError):
            return finding.snippet

        if finding.line_number is None:
            return "\n".join(lines[:50])

        start = max(0, finding.line_number - 10)
        end = min(len(lines), finding.line_number + 10)
        context_lines = []
        for i in range(start, end):
            marker = ">>>" if i + 1 == finding.line_number else "   "
            context_lines.append(f"{marker} {i + 1:4d} | {lines[i]}")
        return "\n".join(context_lines)

    def _parse_response(self, response: str) -> tuple[str, str, str]:
        """Parse LLM response into verdict, confidence, and explanation."""
        verdict = "TRUE_POSITIVE"
        confidence = "MEDIUM"
        explanation = response

        verdict_match = re.search(r"VERDICT:\s*(TRUE_POSITIVE|FALSE_POSITIVE)", response)
        if verdict_match:
            verdict = verdict_match.group(1)

        confidence_match = re.search(r"CONFIDENCE:\s*(HIGH|MEDIUM|LOW)", response)
        if confidence_match:
            confidence = confidence_match.group(1)

        explanation_match = re.search(r"EXPLANATION:\s*(.+?)(?:REMEDIATION:|$)", response, re.DOTALL)
        if explanation_match:
            explanation = explanation_match.group(1).strip()

        return verdict, confidence, explanation



"""Integration tests for CLI commands."""


from typer.testing import CliRunner

from llm_authz_audit.cli import app

runner = CliRunner()


class TestCLIScan:
    def test_scan_vulnerable_dir(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        result = runner.invoke(app, ["scan", str(tmp_path)])
        assert result.exit_code == 1
        assert "SEC001" in result.output

    def test_scan_clean_dir(self, tmp_path):
        (tmp_path / "app.py").write_text('import os\nx = os.environ["KEY"]\n')
        result = runner.invoke(app, ["scan", str(tmp_path)])
        assert result.exit_code == 0

    def test_scan_json_format(self, tmp_path):
        import json
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--format", "json"])
        assert result.exit_code == 1
        data = json.loads(result.output)
        assert "findings" in data
        assert "summary" in data

    def test_scan_nonexistent_dir(self, tmp_path):
        result = runner.invoke(app, ["scan", str(tmp_path / "nonexistent")])
        assert result.exit_code == 2

    def test_scan_verbose(self, tmp_path):
        (tmp_path / "app.py").write_text('x = 1\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "-v"])
        assert "Scanning:" in result.output

    def test_scan_with_exclude(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--exclude", "app.py"])
        assert result.exit_code == 0

    def test_scan_fail_on_critical(self, tmp_path):
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--fail-on", "critical"])
        assert result.exit_code == 1  # SEC001 is critical

    def test_scan_console_shows_banner(self, tmp_path):
        (tmp_path / "app.py").write_text('x = 1\n')
        result = runner.invoke(app, ["scan", str(tmp_path)])
        assert "authz-audit" in result.output
        assert "Analyzers:" in result.output
        assert "Fail on:" in result.output

    def test_scan_json_has_no_banner(self, tmp_path):
        import json
        (tmp_path / "app.py").write_text('x = 1\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--format", "json"])
        assert result.exit_code == 0
        data = json.loads(result.output)
        assert "findings" in data

    def test_scan_quiet_suppresses_banner(self, tmp_path):
        (tmp_path / "app.py").write_text('x = 1\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--quiet"])
        assert "╦" not in result.output
        assert "Analyzers:" not in result.output
        assert "Fail on:" not in result.output

    def test_scan_banner_shows_exclude(self, tmp_path):
        (tmp_path / "app.py").write_text('x = 1\n')
        result = runner.invoke(app, ["scan", str(tmp_path), "--exclude", "tests/*,*.md"])
        assert "Exclude:" in result.output

    def test_scan_min_confidence_filters_low(self, tmp_path):
        # IV001 fires with Confidence.LOW — should be hidden with --min-confidence medium
        code = '''\
from flask import Flask, request
import openai

app = Flask(__name__)

@app.route("/chat", methods=["POST"])
def chat():
    data = request.get_json()
    prompt = data["prompt"]
    return openai.ChatCompletion.create(prompt=prompt)
'''
        (tmp_path / "app.py").write_text(code)
        # Without filter: IV001 should appear
        result = runner.invoke(app, ["scan", str(tmp_path), "--format", "json"])
        import json
        data = json.loads(result.output)
        rule_ids = [f["rule_id"] for f in data["findings"]]
        assert "IV001" in rule_ids

        # With --min-confidence medium: IV001 (LOW confidence) should be hidden
        result2 = runner.invoke(app, ["scan", str(tmp_path), "--format", "json", "--min-confidence", "medium"])
        data2 = json.loads(result2.output)
        rule_ids2 = [f["rule_id"] for f in data2["findings"]]
        assert "IV001" not in rule_ids2

    def test_scan_with_suppress_file(self, tmp_path):
        import json
        (tmp_path / "app.py").write_text('API_KEY = "sk-proj-abcdefghijklmnopqrstuvwxyz1234567890"\n')
        suppress_file = tmp_path / "suppress.yaml"
        suppress_file.write_text(
            'suppressions:\n'
            '  - rule_id: SEC001\n    reason: "Test fixture"\n'
            '  - rule_id: SEC005\n    reason: "Test fixture"\n'
        )
        result = runner.invoke(app, ["scan", str(tmp_path), "--format", "json", "--suppress", str(suppress_file)])
        data = json.loads(result.output)
        rule_ids = [f["rule_id"] for f in data["findings"]]
        assert "SEC001" not in rule_ids
        assert "SEC005" not in rule_ids
        assert result.exit_code == 0


class TestCLIListAnalyzers:
    def test_list_analyzers(self):
        result = runner.invoke(app, ["list-analyzers"])
        assert result.exit_code == 0
        assert "SecretsAnalyzer" in result.output
        assert "EndpointAnalyzer" in result.output
        assert "ToolRBACAnalyzer" in result.output

    def test_list_analyzers_count(self):
        result = runner.invoke(app, ["list-analyzers"])
        # Should list all 13 analyzers
        expected = [
            "SecretsAnalyzer", "EndpointAnalyzer", "ToolRBACAnalyzer",
            "RAGACLAnalyzer", "MCPPermissionAnalyzer", "SessionIsolationAnalyzer",
            "RateLimitingAnalyzer", "OutputFilteringAnalyzer",
            "CredentialForwardingAnalyzer", "AuditLoggingAnalyzer",
            "InputValidationAnalyzer", "PromptInjectionAnalyzer",
            "JSEndpointAnalyzer",
        ]
        for name in expected:
            assert name in result.output


class TestCLIListRules:
    def test_list_rules(self):
        result = runner.invoke(app, ["list-rules"])
        assert result.exit_code == 0
        assert "SEC001" in result.output
        assert "EP001" in result.output

    def test_list_rules_shows_severity(self):
        result = runner.invoke(app, ["list-rules"])
        assert "critical" in result.output
        assert "high" in result.output


class TestCLIInit:
    def test_init(self, tmp_path, monkeypatch):
        monkeypatch.chdir(tmp_path)
        result = runner.invoke(app, ["init"])
        assert result.exit_code == 0
        assert (tmp_path / ".llm-audit.yaml").exists()

    def test_init_already_exists(self, tmp_path, monkeypatch):
        monkeypatch.chdir(tmp_path)
        (tmp_path / ".llm-audit.yaml").write_text("existing")
        result = runner.invoke(app, ["init"])
        assert result.exit_code == 1
